#!/usr/bin/env python2
#Copyright (C) 2015 Mohamed Aziz knani

#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>

from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
from sqlalchemy import Column, Integer, String
from . import app
import time

engine = create_engine(app.config['SQLALCHEMY_DATABASE_URI'])
Base = declarative_base()

Session = sessionmaker(bind=engine)

class Todo(Base):
  __tablename__ = 'Todo'
  id = Column(Integer, primary_key=True)
  owner = Column(Integer)
  todotext = Column(String)
  done = Column(Integer)
  date = Column(String)

  def __init__(self, ow, tt):
    self.owner = ow
    self.todotext = tt
    self.done = 0
    self.date = time.strftime('%Y/%m/%d %H:%M:%S')

  def __str__(self):
    return "(%d, %d, %s, %d, %s)" % (self.id,
                                     self.owner,
                                     self.todotext,
                                     self.done,
                                     self.date)

class User(Base):
  __tablename__ = 'Users'
  id = Column(Integer, primary_key=True)
  username = Column(String)
  password = Column(String)




#!/usr/bin/env python2
#Copyright (C) 2015 Mohamed Aziz knani

#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>

from datetime import datetime
import time
import os
import sys
from urlparse import urljoin
import requests
import json
from os.path import expanduser, join, isfile

reload(sys)
sys.setdefaultencoding('utf8')

def currdir(filename):
  return os.path.join(os.path.dirname(__file__), filename)

def saveUserConfig(data, filename=join(expanduser('~'), 'mytodoconfig.json')):
  d = json.dumps(data)
  with open(filename, 'w') as myfile:
    myfile.write(d)

  return 0

def loadUserConfig(filename=join(expanduser('~'), 'mytodoconfig.json')):
  # Create a default user config if it deosen't exist
  if not isfile(filename):
    saveUserConfig({
      'username' : 'demo',
      'password' : 'test',
      'host'     : 'http://localhost:5000/'
      })

  with open(filename, 'r') as myfile:
    toparse = myfile.read()
  global a
  a = json.loads(toparse)
  if not(a.has_key('username') and a.has_key('password')) or\
      not(a['username'] and a['password']):
    print "Please provide a username and a password in the config file userconfing.json"
    exit()
  return a


def dictToList(sic):
  a = []
  try:
    sic = json.loads(sic)
  except ValueError:
    return False
  else:
    for i in range(len(sic)):
      a.append( sic[str(i)].values() )
    return a

class Client(object):

  def __init__(self, user, password):
    self.cord = {
      'username' : user,
      'password' : password
    }

    self.req = requests.post

  def listall(self):
    c = self.req(urljoin(a['host'], '/api/get/?action=all'), data=self.cord).text
    return dictToList(c)

  def ls(self):
    c = self.req(urljoin(a['host'], '/api/get/?action=ls'), data=self.cord).text
    return dictToList(c)

  def add(self, todotext):
    self.cord['text'] = todotext
    self.req(urljoin(a['host'], '/api/set/?action=add'), data=self.cord)

  def done(self, num):
    self.req(urljoin(a['host'], '/api/set/?action=done&index=%s' % str(num)), data=self.cord)

  def undone(self, num):
    self.req(urljoin(a['host'], '/api/set/?action=undone&index=%s' % str(num)), data=self.cord)

  def remove(self, num):
    self.req(urljoin(a['host'], '/api/set/?action=remove&index=%s' % str(num)), data=self.cord)

  def category(self, tag):
    c = self.req(urljoin(a['host'], '/api/get/?action=search&tag=%s' % tag), data=self.cord).text
    return dictToList(c)

def dy (date) :
  parsed = time.strptime(date, "%Y/%m/%d %H:%M:%S")
  d = (datetime.today() - datetime(parsed.tm_year, parsed.tm_mon, parsed.tm_mday)).days
  return str(d)+' days ago' if d else 'today'




#!/usr/bin/env python2
#Copyright (C) 2015 Mohamed Aziz knani

#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>

from . import database

def checkuser(username, password):
  if username is None or password is None:
    return False
  s = database.Session()
  # Checks if the username exists
  found = s.query(database.User).filter_by(username = username).first()
  if found is None:
    return False

  if found.username != username or found.password != password:
    return False

  return True

def getOwnerbyUsername(username):
  s = database.Session()
  return s.query(database.User).filter_by(username = username).first().id



#!/usr/bin/env python2
#Copyright (C) 2015 Mohamed Aziz knani

#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>

from . import app, database, tools
from flask import jsonify, request, abort

@app.route('/api/get/', methods=['GET', 'POST'])
def apiget():
  t = request.args.get('action')
  if request.method == 'POST':
    username = request.form['username']
    password = request.form['password']
    a = []
    if not tools.checkuser(username, password):
      abort(400)
    s = database.Session()
    if t == 'all':
      a = s.query(database.Todo).filter(database.Todo.owner == tools.getOwnerbyUsername(username)).all()
    elif t == 'ls':
      a = s.query(database.Todo).filter(database.Todo.owner == tools.getOwnerbyUsername(username),
                                        database.Todo.done == 0).all()
    elif t == 'search':
      if request.args.has_key('tag'):
        search = request.args.get('tag')
        a = s.query(database.Todo).filter(database.Todo.owner == tools.getOwnerbyUsername(username),
                                        database.Todo.todotext.like('%'+search+'%')).all()
    # My fast an dirty hack ( This is ugly I know )
    c = dict()
    for i, x in enumerate(a):
      del x.__dict__['_sa_instance_state']
      c[str(i)] = x.__dict__
    return jsonify(c)
  else:
    return 'Send a post request containing username and password'

@app.route('/api/set/', methods=['GET', 'POST'])
def  apiset():
  t = request.args.get('action')
  if request.method == 'POST':
    username = request.form['username']
    password = request.form['password']
    a = []

    if not tools.checkuser(username, password):
      abort(400)

    s = database.Session()

    if request.args.has_key('index'):
      index = int(request.args.get('index'))

      if (t == 'done' or t == 'undone'):
        a = s.query(database.Todo).filter(database.Todo.owner == tools.getOwnerbyUsername(username),
                                          database.Todo.id  == index).first()
        a.done = 1 if t == 'done' else 0
        s.commit()

      if t == 'remove':
        a = s.query(database.Todo).filter(database.Todo.owner == tools.getOwnerbyUsername(username),
                                          database.Todo.id  == index).first()
        if not a is None:
          s.delete(a)
          s.commit()

    if request.form.has_key('text'):
      text = request.form.get('text')
      if t == 'add':
        a = database.Todo(tools.getOwnerbyUsername(username), text)
        s.add(a)
        s.commit()
    return ''
  else:
    return 'Send a post request containing username and password'






from setuptools import setup

setup(
  name='mytodo',
  version='0.2.3',
  description='The free/libre todo list manager',
  url='https://github.com/mohamed-aziz/mytodo',
  author='Mohamed Aziz Knani',

  author_email='medazizknani@gmail.com',

  license='GPL3',
  packages=['mytodo',
            'mytodo.mytodoweb',
            'mytodo.img'],
  scripts=[
    'mytodo/mytodo_cli',
    'mytodo/mytodo_server',
    'mytodo/mytodo_gui'
  ],
  package_data={
    'mytodo.img' : ['close.png', 'done.png']
  },

  install_requires=[
    'requests',
    'flask',
    'sqlalchemy'
  ]
)



#!/usr/bin/env python2
#Copyright (C) 2015 Mohamed Aziz knani

#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>

from flask import Flask

app = Flask(__name__)
from . import config, main


